@@ -1,6 +1,6 @@
 // file kernel/n/c/fftmul.c: multiplication and remainder with FFT
 /*-----------------------------------------------------------------------+
- |  Copyright 2005, Michel Quercia (michel.quercia@prepas.org)           |
+ |  Copyright 2005-2006, Michel Quercia (michel.quercia@prepas.org)      |
  |                                                                       |
  |  This file is part of Numerix. Numerix is free software; you can      |
  |  redistribute it and/or modify it under the terms of the GNU Lesser   |
@@ -128,8 +128,8 @@ static long fftmul_tab[] = {fftmul_lim,
   /* utilisation de FFT */
   else {
     k += 4;
-    f = (l  - l/20 + (6<<k)-1)/(6<<k);
-    i = (k < hw+2) ? 1 : (k==hw+2) ? 2 : 1 << (k-2-hw);
+    f = (l  - l/20 + ((long)6<<k)-1)/((long)6<<k);
+    i = (k < hw+2) ? 1 : (k==hw+2) ? 2 : (long)1 << (k-2-hw);
     n1 = 4*f + 4 + i; n1 -= n1 & (i-1); n1 = xn(fft_improve)(n1,i);
     n2 = 4*f + 2 + i; n2 -= n2 & (i-1); n2 = xn(fft_improve)(n2,i);
     n3 = 4*f     + i; n3 -= n3 & (i-1); n3 = xn(fft_improve)(n3,i);
@@ -142,10 +142,10 @@ static long fftmul_tab[] = {fftmul_lim,
       if (4*f+4 > n2) f = (n2-4)/4;
       if (4*f+6 > n1) f = (n1-6)/4;
     }
-    p = l - (6<<k)*f; if (p < 0) p = 0;
+    p = l - ((long)6<<k)*f; if (p < 0) p = 0;
 #ifdef verbose
     if (l > lsav) {
-      printf(" l=%ld k=%ld f=%ld i=%ld p=%ld n=%ld n=%ld n=%ld ",l,k,f,(long)(1<<k),p,n1,n2,n3);
+      printf(" l=%ld k=%ld f=%ld i=%ld p=%ld n=%ld n=%ld n=%ld ",l,k,f,(long)1<<k,p,n1,n2,n3);
       lsav = l;
     }
 #endif
@@ -195,7 +195,7 @@ static long fftmul_tab[] = {fftmul_lim,
     }
 
     /* recombinaison */
-    xn(sjoin3)(x+p,f,1<<k);
+    xn(sjoin3)(x+p,f,(long)1<<k);
     xn(move)(x+p,l-p,c+p);
     xn(free)(x);
   }
@@ -219,7 +219,6 @@ void xn(fftmul)(chiffre *a, long la, chiffre *b, long lb, chiffre *c) {
       xn(internal_error)("error in fftmul",4,a,la,b,lb,c,la+lb,d,la+lb);
 
   xn(free)(d);
-
 }
 #endif /* debug_fftmul */
 
@@ -323,8 +322,8 @@ static long remdiv_tab[] = {remdiv_lim,
     /* utilisation de FFT */
     else {
       k += 4;
-      f = (l  - l/20 + (6<<k)-1)/(6<<k);
-      i = (k < hw+2) ? 1 : (k==hw+2) ? 2 : 1 << (k-2-hw);
+      f = (l  - l/20 + ((long)6<<k)-1)/((long)6<<k);
+      i = (k < hw+2) ? 1 : (k==hw+2) ? 2 : (long)1 << (k-2-hw);
       n1 = 4*f + 4 + i; n1 -= n1 & (i-1); n1 = xn(fft_improve)(n1,i);
       n2 = 4*f + 2 + i; n2 -= n2 & (i-1); n2 = xn(fft_improve)(n2,i);
       n3 = 4*f     + i; n3 -= n3 & (i-1); n3 = xn(fft_improve)(n3,i);
@@ -337,10 +336,10 @@ static long remdiv_tab[] = {remdiv_lim,
         if (4*f+4 > n2) f = (n2-4)/4;
         if (4*f+6 > n1) f = (n1-6)/4;
       }
-      p = l - (6<<k)*f; if (p < 0) p = 0;
+      p = l - ((long)6<<k)*f; if (p < 0) p = 0;
 #ifdef verbose
       if (l > lsav) {
-        printf(" l=%ld k=%ld f=%ld i=%ld p=%ld n=%ld n=%ld n=%ld ",l,k,f,(long)(1<<k),p,n1,n2,n3);
+        printf(" l=%ld k=%ld f=%ld i=%ld p=%ld n=%ld n=%ld n=%ld ",l,k,f,(long)1<<k,p,n1,n2,n3);
         lsav = l;
       }
 #endif
@@ -378,7 +377,7 @@ static long remdiv_tab[] = {remdiv_lim,
       y = x + ((2*f+2)<<k); xn(ssub)( a,lb+lc, x, y-x);
       z = y + ((2*f+1)<<k); xn(ssub)( a,lb+lc, y, z-y);
       t = z + ((2*f)<<k);   xn(ssub)( a,lb+lc, z, t-z);
-      i = 1 << k;
+      i = (long)1 << k;
     }
 
     if (p) {
@@ -510,8 +509,8 @@ static long fftsqr_tab[] = {fftsqr_lim,
   else {
 
     k += 4;
-    f = (l - l/20 +(6<<k)-1)/(6<<k);
-    i = (k < hw+2) ? 1 : (k==hw+2) ? 2 : 1 << (k-2-hw);
+    f = (l - l/20 +((long)6<<k)-1)/((long)6<<k);
+    i = (k < hw+2) ? 1 : (k==hw+2) ? 2 : (long)1 << (k-2-hw);
     n1 = 4*f + 4 + i; n1 -= n1 & (i-1); n1 = xn(fft_improve)(n1,i);
     n2 = 4*f + 2 + i; n2 -= n2 & (i-1); n2 = xn(fft_improve)(n2,i);
     n3 = 4*f     + i; n3 -= n3 & (i-1); n3 = xn(fft_improve)(n3,i);
@@ -524,10 +523,10 @@ static long fftsqr_tab[] = {fftsqr_lim,
       if (4*f+4 > n2) f = (n2-4)/4;
       if (4*f+6 > n1) f = (n1-6)/4;
     }
-    p = l - (6<<k)*f; if (p < 0) p = 0;
+    p = l - ((long)6<<k)*f; if (p < 0) p = 0;
 #ifdef verbose
     if (l > lsav) {
-      printf(" l=%ld k=%ld f=%ld i=%ld p=%ld n=%ld n=%ld n=%ld ",l,k,f,(long)(1<<k),p,n1,n2,n3);
+      printf(" l=%ld k=%ld f=%ld i=%ld p=%ld n=%ld n=%ld n=%ld ",l,k,f,(long)1<<k,p,n1,n2,n3);
       lsav = l;
     }
 #endif
@@ -573,7 +572,7 @@ static long fftsqr_tab[] = {fftsqr_lim,
     }
 
     /* recombinaison */
-    xn(sjoin3)(x+p,f,1<<k);
+    xn(sjoin3)(x+p,f,(long)1<<k);
     xn(move)(x+p,l-p,b+p);
     xn(free)(x);
   }
@@ -597,7 +596,6 @@ void xn(fftsqr)(chiffre *a, long la, chiffre *b) {
       xn(internal_error)("error in fftsqr",3,a,la,b,2*la,d,2*la);
 
   xn(free)(d);
-
 }
 #endif /* debug_fftmul */
 
@@ -703,8 +701,8 @@ void xn(remsqrt)(chiffre *a, long la, chiffre *b) {
     /* utilisation de FFT */
     else {
       k += 4;
-      f = (l  - l/20 + (6<<k)-1)/(6<<k);
-      i = (k < hw+2) ? 1 : (k==hw+2) ? 2 : 1 << (k-2-hw);
+      f = (l  - l/20 + ((long)6<<k)-1)/((long)6<<k);
+      i = (k < hw+2) ? 1 : (k==hw+2) ? 2 : (long)1 << (k-2-hw);
       n1 = 4*f + 4 + i; n1 -= n1 & (i-1); n1 = xn(fft_improve)(n1,i);
       n2 = 4*f + 2 + i; n2 -= n2 & (i-1); n2 = xn(fft_improve)(n2,i);
       n3 = 4*f     + i; n3 -= n3 & (i-1); n3 = xn(fft_improve)(n3,i);
@@ -717,10 +715,10 @@ void xn(remsqrt)(chiffre *a, long la, chiffre *b) {
         if (4*f+4 > n2) f = (n2-4)/4;
         if (4*f+6 > n1) f = (n1-6)/4;
       }
-      p = l - (6<<k)*f; if (p < 0) p = 0;
+      p = l - ((long)6<<k)*f; if (p < 0) p = 0;
 #ifdef verbose
       if (l > lsav) {
-        printf(" l=%ld k=%ld f=%ld i=%ld p=%ld n=%ld n=%ld n=%ld ",l,k,f,(long)(1<<k),p,n1,n2,n3); fflush(stdout);
+        printf(" l=%ld k=%ld f=%ld i=%ld p=%ld n=%ld n=%ld n=%ld ",l,k,f,(long)1<<k,p,n1,n2,n3); fflush(stdout);
         lsav = l;
       }
 #endif
@@ -755,7 +753,7 @@ void xn(remsqrt)(chiffre *a, long la, chiffre *b) {
       y = x + ((2*f+2)<<k); xn(ssub)( a,la, x, y-x);
       z = y + ((2*f+1)<<k); xn(ssub)( a,la, y, z-y);
       t = z + ((2*f)<<k);   xn(ssub)( a,la, z, t-z);
-      i = 1 << k;
+      i = (long)1 << k;
     }
 
     if (p) {
