@@ -1,6 +1,6 @@
 // file kernel/n/c/gcd.c: greatest common divisor
 /*-----------------------------------------------------------------------+
- |  Copyright 2005, Michel Quercia (michel.quercia@prepas.org)           |
+ |  Copyright 2005-2006, Michel Quercia (michel.quercia@prepas.org)      |
  |                                                                       |
  |  This file is part of Numerix. Numerix is free software; you can      |
  |  redistribute it and/or modify it under the terms of the GNU Lesser   |
@@ -122,7 +122,7 @@ static void divise
     aa = cc + *la - lb + 1;
 
     /* dcalage  appliquer  b+r pour avoir le bit de poids fort  1 */
-    for (bb=b[lb-1], n=0; (bb & (BASE/2)) == 0; bb <<=1, n++);
+    for (bb=b[lb-1], n=0; (bb & (BASE_2)) == 0; bb <<=1, n++);
     if (n) {
         aa[(*la)++] = xn(shift_up)(a,*la,aa,n);
         xn(shift_up)(b,lb,b,n);
@@ -359,74 +359,315 @@ static void mulmat(chiffre **_v0, long *_l0, chiffre **_v1, long *_l1) {
         
 /*
   entre :
-    a > b > 0 avec a < BASE*b
-    x = tableau de 4 chiffres [p,s,q,r]
+    x = tableau de 8 chiffres [a0,a1,b0,b1,p,s,q,r]
+    a = a0 + BASE*a1, b = b0 + BASE*b1
+
+  contrainte : 0 < b < a < BASE*b
 
   Dveloppe en fraction continue la fraction a/b tant que les coefficients
   tiennent sur un chiffre
 
   sortie :
-   [p,s,q,r] <- coefficients des combinaisons effectues
+   [a0,a1,b0,b1] <- ind.
+   [p,s,q,r]     <- coefficients des combinaisons effectues
 */
-void xn(gcd_2)(ndouble a, ndouble b, chiffre *x)
-#ifdef assembly_sn_gcd_2
-;
-#else
+void xn(gcd_2)(chiffre *x)
+#if defined(assembly_sn_gcd_2)
+; 
+#elif defined(use_clong)
 {
-    ndouble p,q,r,s,t;
+    ndouble a,b,p,q,r,s,t;
 
-    x[0] = x[1] = 1; x[2] = x[3] = 0;
+    a = (ndouble)x[0] + ((ndouble)x[1] << HW);
+    b = (ndouble)x[2] + ((ndouble)x[3] << HW);
+    x[4] = x[5] = 1; x[6] = x[7] = 0;
     p = s = 1;       q = r = 0;
     do {
         t = a/b; a -= t*b; q += t*p; s += t*r;
-        if ((q >= BASE) || (s >= BASE)) break;
-        x[2] = q; x[1] = s;
+        if ((q >= 2*BASE_2) || (s >= 2*BASE_2)) break;
+        x[6] = q; x[5] = s;
         if (a == 0) break;
         t = b/a; b -= t*a; p += t*q; r += t*s;
-        if ((p >= BASE) || (r >= BASE)) break;
-        x[0] = p; x[3] = r;
+        if ((p >= 2*BASE_2) || (r >= 2*BASE_2)) break;
+        x[4] = p; x[7] = r;
         if (b == 0) break;
     } while(1);
 }
-#endif /* assembly_sn_gdd_2 */
+#else /* mode dlong/slong : division par soustraction/dcalage */
+{
+#define a0 x[0]
+#define a1 x[1]
+#define b0 x[2]
+#define b1 x[3]
+#define p  x[4]
+#define q  x[6]
+#define r  x[7]
+#define s  x[5]
+
+  chiffre t;
+  long i;
+
+  /* [p,q,r,s] <- Id */
+  p = s = 1; q = r = 0;
+
+  while(1) {
+
+    /* Ici a >= b. Dcale b,p,r tant que a >= 2*b */
+    a1 = a1 - b1 - (a0 < b0);
+    a0 -= b0;
+    for (i=0; ;i++) {
+      t = b1 + (a0 < b0); a0 -= b0;
+      if (a1 < t) {a1 -= t; break;}
+      a1 -= t;
+      b1 = 2*b1 + (b0 >= (BASE_2));
+      b0 <<= 1;
+      if ((p & (BASE_2)) || (r & (BASE_2))) {p >>= i; r >>= i; return;}
+      p <<= 1;
+      r <<= 1;
+    }
+
+    /* Ici a0:a1 contient a - 2^(i+1)*b,
+           b0:b1 contient 2^i*b
+	   p     contient 2^i*p
+	   r     contient 2^i*r
+       Calcule a/b par soustractions et dcalages et sort si q ou s dborde */
+    a0 += b0; a1 += b1 + (a0 < b0);
+    q += p; s += r;
+    if ((q < p) || (s < r)) {q -= p; s -= r; p >>= i; r >>= i; return;}
+    while (i) {
+      b0 >>= 1; if (b1&1) b0 += BASE_2;
+      b1 >>= 1;
+      p  >>= 1;
+      r  >>= 1;
+      i--;
+      if ((a1 > b1) || ((a1 == b1) && (a0 >= b0))) {
+	a1 -= b1 + (a0 < b0); a0 -= b0;
+	q += p; s += r;
+	if ((q < p) || (s < r)) {q -= p; s -= r; p >>= i; r >>= i; return;}
+      }
+    }
+
+    /* Fin de la division de a par b.
+       Ici a0:a1 contient le reste, b,p,r ont t restaurs et q,s mis  jour.
+       si a = 0, c'est termin */
+    if ((a0 == 0) && (a1 == 0)) return;
+
+    /* Ici b >= a. Dcale a,s,q tant que b >= 2*a */
+    b1 = b1 - a1 - (b0 < a0);
+    b0 -= a0;
+    for (i=0; ;i++) {
+      t = a1 + (b0 < a0); b0 -= a0;
+      if (b1 < t) {b1 -= t; break;}
+      b1 -= t;
+      a1 = 2*a1 + (a0 >= (BASE_2));
+      a0 <<= 1;
+      if ((s & (BASE_2)) || (q & (BASE_2))) {s >>= i; q >>= i; return;}
+      s <<= 1;
+      q <<= 1;
+    }
+
+    /* Ici b0:b1 contient b - 2^(i+1)*a,
+           a0:a1 contient 2^i*a
+	   s     contient 2^i*s
+	   q     contient 2^i*q
+       Calcule b/a par soustractions et dcalages et sort si p ou r dborde */
+    b0 += a0; b1 += a1 + (b0 < a0);
+    r += s; p += q;
+    if ((r < s) || (p < q)) {r -= s; p -= q; s >>= i; q >>= i; return;}
+    while(i) {
+      a0 >>= 1; if (a1&1) a0 += BASE_2;
+      a1 >>= 1;
+      s  >>= 1;
+      q  >>= 1;
+      i--;
+      if ((b1 > a1) || ((b1 == a1) && (b0 >= a0))) {
+	b1 -= a1 + (b0 < a0); b0 -= a0;
+	r += s; p += q;
+	if ((r < s) || (p < q)) {r -= s; p -= q; s >>= i; q >>= i; return;}
+      }
+    }
+
+    /* Fin de la division de b par a.
+       Ici b0:b1 contient le reste, a,s,q ont t restaurs et r,p mis  jour.
+       si b = 0, c'est termin */
+    if ((b0 == 0) && (b1 == 0)) return;
+
+  } /* while(1) */
+
+#undef a0
+#undef a1
+#undef b0
+#undef b1
+#undef p
+#undef q
+#undef r
+#undef s
+}
+#endif /* assembly_sn_gdd_2, use_clong */
 
                      /* +-----------------------------+
                         |  Demi-pgcd  deux chiffres  |
                         +-----------------------------+ */
  
 /*
- entre :
-   a > b > 0
-   x = tableau de 4 chiffres [p,s,q,r]
+  entre :
+    x = tableau de 8 chiffres [a0,a1,b0,b1,p,s,q,r]
+    a = a0 + BASE*a1, b = b0 + BASE*b1
 
- Dveloppe en fraction continue les fractions a/(b+1) et (a+1)/b
- tant que les quotients concident et que les coefficients tiennent
- sur un chiffre
+  contrainte : 0 < b < a
 
- sortie :
-   [p,s,q,r] <- coefficients des combinaisons effectues
+  Dveloppe en fraction continue les fractions a/(b+1) et (a+1)/b
+  tant que les quotients concident et que les coefficients tiennent
+  sur un chiffre
+
+  sortie :
+    [a0,a1,b0,b1] <- ind.
+    [p,s,q,r]     <- coefficients des combinaisons effectues
 */
-void xn(hgcd_2)(ndouble a, ndouble b, chiffre *x)
-#ifdef assembly_sn_hgcd_2
+void xn(hgcd_2)(chiffre *x)
+#if defined(assembly_sn_hgcd_2)
 ;
-#else
+#elif defined(use_clong)
 {
-    ndouble p,q,r,s,t;
+    ndouble a,b,p,q,r,s,t;
 
-    x[0] = x[1] = 1; x[2] = x[3] = 0;
+    a = (ndouble)x[0] + ((ndouble)x[1] << HW);
+    b = (ndouble)x[2] + ((ndouble)x[3] << HW);
+    x[4] = x[5] = 1; x[6] = x[7] = 0;
     p = s = 1;       q = r = 0;
     do {
         t = (a-q)/(b+p); if (t == 0) break;
         a -= t*b; q += t*p; s += t*r;
-        if ((q >= BASE) || (s >= BASE)) break;
-        x[2] = q; x[1] = s;
+        if ((q >= 2*BASE_2) || (s >= 2*BASE_2)) break;
+        x[6] = q; x[5] = s;
         t = (b-r)/(a+s); if (t == 0) break;
         b -= t*a; p += t*q; r += t*s;
-        if ((p >= BASE) || (r >= BASE)) break;
-        x[0] = p; x[3] = r;
+        if ((p >= 2*BASE_2) || (r >= 2*BASE_2)) break;
+        x[4] = p; x[7] = r;
     } while(1);
 }
-#endif /* assembly_sn_hgcd_2 */
+#else /* mode dlong/slong : division par soustraction/dcalage */
+{
+#define a0 x[0]
+#define a1 x[1]
+#define b0 x[2]
+#define b1 x[3]
+#define p  x[4]
+#define q  x[6]
+#define r  x[7]
+#define s  x[5]
+
+  chiffre t;
+  long i;
+
+  /* [p,q,r,s] <- Id */
+  p = s = 1; q = r = 0;
+
+  /* a <- a-q, b <- b+p */
+  b0++;  b1 += (b0 == 0);
+
+  while(1) {
+
+    /* Ici a-q >= b+p. Dcale b,p,r tant que a-q >= 2*(b+p) */
+    a1 = a1 - b1 - (a0 < b0);
+    a0 -= b0;
+    for (i=0; ;i++) {
+      t = b1 + (a0 < b0); a0 -= b0;
+      if (a1 < t) {a1 -= t; break;}
+      a1 -= t;
+      b1 = 2*b1 + (b0 >= (BASE_2));
+      b0 <<= 1;
+      if ((p & (BASE_2)) || (r & (BASE_2))) {p >>= i; r >>= i; return;}
+      p <<= 1;
+      r <<= 1;
+    }
+
+    /* Ici a0:a1 contient a-q - 2^(i+1)*(b+p),
+           b0:b1 contient 2^i*(b+p)
+	   p     contient 2^i*p
+	   r     contient 2^i*r
+       Calcule (a-q)/(b+p) par soustractions et dcalages et sort si q ou s dborde */
+    a0 += b0; a1 += b1 + (a0 < b0);
+    q += p; s += r;
+    if ((q < p) || (s < r)) {q -= p; s -= r; p >>= i; r >>= i; return;}
+    while (i) {
+      b0 >>= 1; if (b1&1) b0 += BASE_2;
+      b1 >>= 1;
+      p  >>= 1;
+      r  >>= 1;
+      i--;
+      if ((a1 > b1) || ((a1 == b1) && (a0 >= b0))) {
+	a1 -= b1 + (a0 < b0); a0 -= b0;
+	q += p; s += r;
+	if ((q < p) || (s < r)) {q -= p; s -= r; p >>= i; r >>= i; return;}
+      }
+    }
+    a0 += q; a1 += (a0 < q);
+    b1 -= (b0 < p); b0 -= p;
+
+    /* Fin de la division de a-q par b+p.
+       Ici a0:a1 contient le reste, b,p,r ont t restaurs et q,s mis  jour.
+       si a+s > b-r, c'est termin */
+    a0 += s; a1 += (a0 < s);
+    b1 -= (b0 < r); b0 -= r;
+    if ((a1 > b1) || ((a1 == b1) && (a0 > b0))) return;
+
+    /* Ici b-r >= a+s. Dcale a,s,q tant que b-r >= 2*(a+s) */
+    b1 = b1 - a1 - (b0 < a0);
+    b0 -= a0;
+    for (i=0; ;i++) {
+      t = a1 + (b0 < a0); b0 -= a0;
+      if (b1 < t) {b1 -= t; break;}
+      b1 -= t;
+      a1 = 2*a1 + (a0 >= (BASE_2));
+      a0 <<= 1;
+      if ((s & (BASE_2)) || (q & (BASE_2))) {s >>= i; q >>= i; return;}
+      s <<= 1;
+      q <<= 1;
+    }
+
+    /* Ici b0:b1 contient b-r - 2^(i+1)*(a+s),
+           a0:a1 contient 2^i*(a+s)
+	   s     contient 2^i*s
+	   q     contient 2^i*q
+       Calcule (b-r)/(a+s) par soustractions et dcalages et sort si p ou r dborde */
+    b0 += a0; b1 += a1 + (b0 < a0);
+    r += s; p += q;
+    if ((r < s) || (p < q)) {r -= s; p -= q; s >>= i; q >>= i; return;}
+    while(i) {
+      a0 >>= 1; if (a1&1) a0 += BASE_2;
+      a1 >>= 1;
+      s  >>= 1;
+      q  >>= 1;
+      i--;
+      if ((b1 > a1) || ((b1 == a1) && (b0 >= a0))) {
+	b1 -= a1 + (b0 < a0); b0 -= a0;
+	r += s; p += q;
+	if ((r < s) || (p < q)) {r -= s; p -= q; s >>= i; q >>= i; return;}
+      }
+    }
+    b0 += r; b1 += (b0 < r);
+    a1 -= (a0 < s); a0 -= s;
+
+    /* Fin de la division de b-r par a+s.
+       Ici b0:b1 contient le reste, a,s,q ont t restaurs et r,p mis  jour.
+       si b+p > a-q, c'est termin */
+    b0 += p; b1 += (b0 < p);
+    a1 -= (a0 < q); a0 -= q;
+    if ((b1 > a1) || ((b1 == a1) && (b0 > a0))) return;
+
+  } /* while(1) */
+
+#undef a0
+#undef a1
+#undef b0
+#undef b1
+#undef p
+#undef q
+#undef r
+#undef s
+}
+#endif /* assembly_sn_hgcd_2, use_clong */
 
                          /* +--------------------+
                             |  Pgcd quadratique  |
@@ -447,7 +688,7 @@ void xn(hgcd_2)(ndouble a, ndouble b, chiffre *x)
   ou les fractions a/(b+1) et (a+1)/b tant que les quotients sont
   gaux et la prcision des nombres rsiduels est suffisante (mode = 2).
 
-  dans les trois mode, calcule des entiers naturels p',q',r',s',a',b' tels que
+  dans les trois modes, calcule des entiers naturels p',q',r',s',a',b' tels que
     a = p'*a' + q'*b'
     b = r'*a' + s'*b'
     p'*s' - q'*r' = 1
@@ -478,15 +719,18 @@ void xn(gcd_n2)
 ;
 #else
 {
-    chiffre *a, *b, *p, *q, *r, *s, *u, *v, x[4];
+    chiffre *a, *b, *p, *q, *r, *s, *u, *v, x[8];
     long    la, lb, lp, lq, lr, ls, lu, lv, n;
-    ndouble a0,b0;
     long i,j,l,t;
 
-#define p0 x[0]
-#define q0 x[2]
-#define r0 x[3]
-#define s0 x[1]
+#define a0 x[0]
+#define a1 x[1]
+#define b0 x[2]
+#define b1 x[3]
+#define p0 x[4]
+#define q0 x[6]
+#define r0 x[7]
+#define s0 x[5]
 
     /* mmoire de travail : 2*max(la,lb)+2 chiffres */
     n = (_l[0] > _l[1]) ? _l[0] : _l[1];
@@ -509,8 +753,9 @@ void xn(gcd_n2)
         t = la - lb;
 
         /* Extrait autant de bits de poids fort de a et b que possible.
-           La premire division qui sera effectue est a0/b0 si on est en
-           mode exact, a0/(b0+1) sinon ; il faut garantir que le quotient
+	   Soient aa = a0 + BASE*a1 et bb = b0 + BASE*b1 les ttes de a et b.
+           La premire division qui sera effectue est aa/bb si on est en
+           mode exact, aa/(bb+1) sinon ; il faut garantir que le quotient
            obtenu est non nul et tient sur un chiffre.
 
            Cas a[la-1] > b[lb-1] :
@@ -521,7 +766,7 @@ void xn(gcd_n2)
 
            Cas a[la-1] = b[lb-1] et t = 0 :
               on prend autant de chiffres pour a et b
-              il se peut qu'on obtienne a0 = b0 en mode inexact, prvoir
+              il se peut qu'on obtienne aa = bb en mode inexact, prvoir
               de produire un quotient gal  1 dans ce cas.
 
            Cas a[la-1] = b[lb-1] et t > 0 :
@@ -529,46 +774,65 @@ void xn(gcd_n2)
               dans les autres cas on prend un chiffre de plus pour a
         */
         if (lb-l == 1) {
-            a0 = (ndouble)a[la-1];
-            b0 = (ndouble)b[lb-1];
-            if ((a0 < b0) || ((a0 == b0) && (t) && (mode == 2))) {
-                a0 = (a0 << HW) + (ndouble)a[la-2];
-                t--;
-            }
+	  a0 = a[la-1]; a1 = 0;
+          b0 = b[lb-1]; b1 = 0;
+	  if ((a0 < b0) || ((a0 == b0) && (t) && (mode == 2))) {
+	    a1 = a0; a0 = a[la-2];
+	    t--;
+	  }
         }
         else {
-            a0 = ((ndouble)a[la-1] << HW) + (ndouble)a[la-2];
-            b0 = ((ndouble)b[lb-1] << HW) + (ndouble)b[lb-2];
-            if ((a0 < b0) || ((a0 == b0) && (t))) {
-                for (j=HW; (zdouble)a0 >= 0; j--, a0 <<= 1);
-                a0 += (ndouble)a[la-3] >> j;
-                b0 >>= j;
-                t--;
-            }
-            else if (lb-l > 2) {
-                for (j=HW; (zdouble)a0 >= 0; j--, a0 <<= 1);
-                a0 += (ndouble)a[la-3] >> j;
-                b0  = (b0 << (HW-j)) + ((ndouble)b[lb-3] >> j);
-            }
+	  a0 = a[la-2]; a1 = a[la-1];
+          b0 = b[lb-2]; b1 = b[lb-1];
+	  if ((a1 < b1) || ((a1 == b1) && ((a0 < b0) || ((a0 == b0) && (t))))) {
+	    for (j=HW; (a1 & (BASE_2)) == 0; j--, a1 <<= 1);
+	    if (j < HW) {
+	      a1 += a0 >> j; a0 = (a0 << (HW-j)) + (a[la-3] >> j);
+	      b0 = (b1 << (HW-j)) + (b0 >> j); b1 >>= j;
+	    } else {
+	      b0 = b1; b1 = 0;
+	    }
+	    t--;
+	  }
+	  else if (lb-l > 2) {
+	    for (j=HW; (a1 & (BASE_2)) == 0; j--, a1 <<= 1);
+	    if (j < HW) {
+	      a1 += a0 >> j; a0 = (a0 << (HW-j)) + (a[la-3] >> j);
+	      b1 = (b1 << (HW-j)) + (b0 >> j);
+	      b0 = (b0 << (HW-j)) + (b[lb-3] >> j);
+	    }
+	  }
         }
 
         /* si on a charg tous les bits de a et b et mode < 2, effectue autant
            de divisions que possible tant que les coefficients des combinaisons
            tiennent dans un chiffre */
-        if ((mode != 2) && (t==0) && (la <= 2)) xn(gcd_2)(a0,b0,x);
+        if ((mode != 2) && (t==0) && (la <= 2)) xn(gcd_2)(x);
 
-        /* si a0 = b0, simule une unique division avec quotient = 1.
-           Ceci comprend le cas douteux a[la-1]=b[la-1], t=0, mode inexact */
-        else if (a0 == b0) {q0 = 1; r0 = 0;}
+	/* si aa = bb, q0 <- 1 */
+	else if ((a1 == b1) && (a0 == b0)) {q0 = 1; r0 = 0;}
+	
+        /* si t > 0, divise aa par bb+1 */
+        else if (t) {
+#ifdef ndouble
+          ndouble aa = ((ndouble)a1 << HW) + (ndouble)a0;
+          ndouble bb = ((ndouble)b1 << HW) + (ndouble)b0 + 1;
+	  q0 = aa/bb;
+#else
+	  while (b1) {
+	    b0 = (b1 << (HW-1)) + (b0 >> 1); b1 >>= 1;
+	    a0 = (a1 << (HW-1)) + (a0 >> 1); a1 >>= 1;
+	  }
+	  b0++;
+	  if (b0) xn(div_0)(a0,a1,b0,&q0,&r0); else q0 = a1;
+#endif
+	  r0 = 0;
+	}
 
-        /* si a0 > b0 et t > 0, divise a0 par b0+1.
-          Rmq :  b0+1 ne peut pas dborder ici */
-        else if (t) {q0 = a0/(b0+1); r0 = 0;}
-
-        /* dernier cas : a et b comportent des bits ngligs, t = 0 et a0 > b0.
-           Dcompose a0/(b0+1) et (a0+1)/b0 en fraction continue tant que les
+        /* dernier cas : a et b comportent des bits ngligs, t = 0 et aa > bb.
+           Dcompose aa/(bb+1) et (aa+1)/bb en fraction continue tant que les
            coefficients tiennent dans un chiffre et les quotients sont gaux */
-        else xn(hgcd_2)(a0,b0,x);
+        else xn(hgcd_2)(x);
 
         /*
           mise  jour a,b,p,q,r,s.
@@ -630,12 +894,12 @@ void xn(gcd_n2)
                 else for (j=lr+1; (j<lb) && (b[j-1] == 0); j++);
                 if (l < j) l = j;
 
-                if ((ls <= la) && (a[ls-1] < BASE-1-s[ls-1])) j = ls;
-                else for (j=ls+1; (j<la) && (a[j-1] == BASE-1); j++);
+                if ((ls <= la) && (a[ls-1] < (BASE_2)+(BASE_2-1)-s[ls-1])) j = ls;
+                else for (j=ls+1; (j<la) && (a[j-1] == (BASE_2)+(BASE_2-1)); j++);
                 if (l < j) l = j;
 
-                if ((lp <= lb) && (b[lp-1] < BASE-1-p[lp-1])) j = lp;
-                else for (j=lp+1; (j<lp) && (p[j-1] == BASE-1); j++);
+                if ((lp <= lb) && (b[lp-1] < (BASE_2)+(BASE_2-1)-p[lp-1])) j = lp;
+                else for (j=lp+1; (j<lp) && (p[j-1] == (BASE_2)+(BASE_2-1)); j++);
                 if (l < j) l = j;
 
             }
@@ -647,6 +911,10 @@ void xn(gcd_n2)
     /* termin */
     xn(free_tmp)(u);
 
+#undef a0
+#undef a1
+#undef b0
+#undef b1
 #undef p0
 #undef q0
 #undef r0
@@ -732,12 +1000,12 @@ void xn(lehmer)
         else for (j=lr+1; (j<lb) && (b[j-1] == 0); j++);
         if (n < j) n = j;
         
-        if ((ls <= la) && (a[ls-1] < BASE-1-s[ls-1])) j = ls;
-        else for (j=ls+1; (j<la) && (a[j-1] == BASE-1); j++);
+        if ((ls <= la) && (a[ls-1] < (BASE_2)+(BASE_2-1)-s[ls-1])) j = ls;
+        else for (j=ls+1; (j<la) && (a[j-1] == (BASE_2)+(BASE_2-1)); j++);
         if (n < j) n = j;
         
-        if ((lp <= lb) && (b[lp-1] < BASE-1-p[lp-1])) j = lp;
-        else for (j=lp+1; (j<lp) && (p[j-1] == BASE-1); j++);
+        if ((lp <= lb) && (b[lp-1] < (BASE_2)+(BASE_2-1)-p[lp-1])) j = lp;
+        else for (j=lp+1; (j<lp) && (p[j-1] == (BASE_2)+(BASE_2-1)); j++);
         if (n < j) n = j;
         
     }
