@@ -1,6 +1,6 @@
 // file kernel/n/c/timing-gmp.c: timing of gmp-functions
 /*-----------------------------------------------------------------------+
- |  Copyright 2005, Michel Quercia (michel.quercia@prepas.org)           |
+ |  Copyright 2005-2006, Michel Quercia (michel.quercia@prepas.org)      |
  |                                                                       |
  |  This file is part of Numerix. Numerix is free software; you can      |
  |  redistribute it and/or modify it under the terms of the GNU Lesser   |
@@ -266,7 +266,7 @@ test_t test[] = {
                             |  Variables globales  |
                             +----------------------+ */
 
-#define HW 32                        /* taille mot */
+#define HW (8*sizeof(long))          /* taille mot */
 gmp_randstate_t gmp_randstate;       /* gnrateur alatoire de gmp */
 
 
@@ -346,10 +346,10 @@ void chronometre(int t, long *m, long *n, double temps, long bits, double unit)
     if (t1 < 0) {
       /* dbordement de l'horloge. Avec un peu de chance a n'a dbord
          qu'une fois, */
-      t1 += 1 << (8*sizeof(clock_t)-2);
-      t1 += 1 << (8*sizeof(clock_t)-2);
-      t1 += 1 << (8*sizeof(clock_t)-2);
-      t1 += 1 << (8*sizeof(clock_t)-2);
+      t1 += 1L << (8*sizeof(clock_t)-2);
+      t1 += 1L << (8*sizeof(clock_t)-2);
+      t1 += 1L << (8*sizeof(clock_t)-2);
+      t1 += 1L << (8*sizeof(clock_t)-2);
     }
 
     /* libre la mmoire */
@@ -497,7 +497,7 @@ int main(int argc, char **argv) {
   }
 
   /* initialisation des gnrateurs alatoires */
-  srand(seed);
+  srandom(seed);
   gmp_randinit_default(gmp_randstate);
   gmp_randseed_ui(gmp_randstate,seed);
 
@@ -511,14 +511,14 @@ int main(int argc, char **argv) {
       }
   }
 
-#define PRINTF(...) do {printf(__VA_ARGS__); if (res) fprintf(fres,__VA_ARGS__);} while (0)
-
   /* tests pour une seule longueur */
   if (!range) {
     for (i=0; i < nbtests; i++) if (test[i].sel) {
-      PRINTF("%-16s", test[i].name);
+      printf("%-16s", test[i].name);
+      if (res) fprintf(fres,"%-16s", test[i].name);
       chronometre(i,&m,&n,temps,bits,unit);
-      PRINTF("%10ld (%ld)\n", m,n);
+      printf("%10ld (%ld)\n", m,n);
+      if (res) fprintf(fres,"%10ld (%ld)\n", m,n);
     }
   }
 
@@ -526,22 +526,35 @@ int main(int argc, char **argv) {
   else {
 
     /* recopie la ligne de commande */
-    PRINTF("#");
-    for (j=argc, s=argv; j; j--, s++) PRINTF(" %s",*s);
-    PRINTF("\n"); fflush(stdout);
+    printf("#");
+    for (j=argc, s=argv; j; j--, s++) printf(" %s",*s);
+    printf("\n"); fflush(stdout);
+    if (res) {
+      fprintf(fres,"#");
+      for (j=argc, s=argv; j; j--, s++) fprintf(fres," %s",*s);
+      fprintf(fres,"\n");
+    }
 
     /* noms des algorithmes */
-    PRINTF("#           bits");
-    for (i=0; i < nbtests; i++) if (test[i].sel) PRINTF("%16s", test[i].name);
-    PRINTF("\n");
+    printf("#           bits");
+    for (i=0; i < nbtests; i++) if (test[i].sel) printf("%16s", test[i].name);
+    printf("\n");
+    if (res) {
+      fprintf(fres,"#           bits");
+      for (i=0; i < nbtests; i++) if (test[i].sel) fprintf(fres,"%16s", test[i].name);
+      fprintf(fres,"\n");
+    }
 
     for (bits = start; bits <= stop; bits += step) {
-      PRINTF("%16ld", bits);
+      printf("%16ld", bits);
+      if (res) fprintf(fres,"%16ld", bits);
       for (i=0; i < nbtests; i++) if (test[i].sel) {
         chronometre(i,&m,&n,temps,bits,unit);
-        PRINTF("%16ld",m);
+        printf("%16ld",m);
+        if (res) fprintf(fres,"%16ld",m);
       }
-      PRINTF("\n");
+      printf("\n");
+      if (res) fprintf(fres,"\n");
     }
   }
 
