@@ -1,6 +1,6 @@
 // file kernel/n/c/timing.c: timing of n-functions
 /*-----------------------------------------------------------------------+
- |  Copyright 2005, Michel Quercia (michel.quercia@prepas.org)           |
+ |  Copyright 2005-2006, Michel Quercia (michel.quercia@prepas.org)      |
  |                                                                       |
  |  This file is part of Numerix. Numerix is free software; you can      |
  |  redistribute it and/or modify it under the terms of the GNU Lesser   |
@@ -62,6 +62,42 @@ void xn(calc_but3)(chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiffre
 }
 
 
+                        /* +--------------------------+
+                           |  Combinaison de rsidus  |
+                           +--------------------------+ */
+
+void xn(size_join)(long *la, long *lb, long *lc, long *ld) {
+  long h,k,n,p,q;
+
+  if (*la < 40) *la = 40;
+  k = *la/20;
+  h = *la/(6*k)-1;
+  n = (2*h+2)*k;
+  p = (2*h+1)*k;
+  q = (2*h)*k;
+
+  *lb = -n - p - q;
+  *lc = 0;
+  *ld = 0;
+}
+
+void xn(calc_join)(chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiffre *d) {
+  long h,k,n,p,q;
+
+  k = la/20;
+  h = la/(6*k)-1;
+  n = (2*h+2)*k;
+  p = (2*h+1)*k;
+  q = (2*h)*k;
+
+  xn(clear)(b,n+p+q);
+  xn(ssub)(a,la,b,n);
+  xn(ssub)(a,la,b+n,p);
+  xn(ssub)(a,la,b+n+p,q);
+
+  xn(sjoin3)(b,h,k);  
+}
+
                         /* +------------------------+
                            |  Multiplication/Carr  |
                            +------------------------+ */
@@ -238,8 +274,8 @@ void xn(size_inv)(long *la, long *lb, long *lc, long *ld) {
 void xn(calc_div_n2)
 (chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiffre *d) {
   
-  b[lb-1] |= (BASE/2);
-  if ((ndouble)a[la-1] >= (ndouble)b[lb-1]) a[la-1] = b[lb-1] - 1;
+  b[lb-1] |= (BASE_2);
+  if (a[la-1] >= b[lb-1]) a[la-1] = b[lb-1] - 1;
   xn(move)(a,la,d);
   xn(div_n2)(d,la-lb,b,lb,c);
 
@@ -251,7 +287,7 @@ void xn(calc_div_n2)
 (chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiffre *d) {
   long l = la-lb;
 
-  b[lb-1] |= (BASE/2);
+  b[lb-1] |= (BASE_2);
   if (l < lb) xn(toommul)(b,lb,a,l,d); else xn(toommul)(a,l,b,lb,d);
   xn(dec1)(d,la);
   xn(div_n2)(d,la-lb,b,lb,c);
@@ -266,8 +302,8 @@ void xn(calc_div_n2)
 void xn(calc_burnidiv)
 (chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiffre *d) {
   
-  b[lb-1] |= (BASE/2);
-  if ((ndouble)a[la-1] >= (ndouble)b[lb-1]) a[la-1] = b[lb-1] - 1;
+  b[lb-1] |= (BASE_2);
+  if (a[la-1] >= b[lb-1]) a[la-1] = b[lb-1] - 1;
   xn(move)(a,la,d);
   xn(burnidiv)(d,la-lb,b,lb,c);
 
@@ -279,7 +315,7 @@ void xn(calc_burnidiv)
 (chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiffre *d) {
   long l = la-lb;
 
-  b[lb-1] |= (BASE/2);
+  b[lb-1] |= (BASE_2);
   if (l > 2) xn(clear)(a,l/2);
   if (l < lb) xn(toommul)(b,lb,a,l,d); else xn(toommul)(a,l,b,lb,d);
   xn(dec1)(d,la);
@@ -293,7 +329,7 @@ void xn(calc_burnidiv)
 void xn(calc_modinv)(chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiffre *d) {
   chiffre *x;
   
-  a[la-1] |= (BASE/2);
+  a[la-1] |= (BASE_2);
   x = xn(alloc)(2*la+1);
   xn(clear)(x,2*la); x[2*la] = 1;
   xn(moddiv)(x,la+1,a,la,b,0);
@@ -306,8 +342,8 @@ void xn(calc_modinv)(chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiff
 void xn(calc_moddivnr)
 (chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiffre *d) {
   
-  b[lb-1] |= (BASE/2);
-  if ((ndouble)a[la-1] >= (ndouble)b[lb-1]) a[la-1] = b[lb-1] - 1;
+  b[lb-1] |= (BASE_2);
+  if (a[la-1] >= b[lb-1]) a[la-1] = b[lb-1] - 1;
   xn(move)(a,la,d);
   xn(moddiv)(d,la-lb,b,lb,c,0);
 
@@ -316,8 +352,8 @@ void xn(calc_moddivnr)
 void xn(calc_moddivr)
 (chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiffre *d) {
   
-  b[lb-1] |= (BASE/2);
-  if ((ndouble)a[la-1] >= (ndouble)b[lb-1]) a[la-1] = b[lb-1] - 1;
+  b[lb-1] |= (BASE_2);
+  if (a[la-1] >= b[lb-1]) a[la-1] = b[lb-1] - 1;
   xn(move)(a,la,d);
   xn(moddiv)(d,la-lb,b,lb,c,1);
 
@@ -329,7 +365,7 @@ void xn(calc_moddivnr)
 (chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiffre *d) {
   long l = la-lb;
 
-  b[lb-1] |= (BASE/2);
+  b[lb-1] |= (BASE_2);
   if (l > 2) xn(clear)(a,l/2);
   if (l < lb) xn(fftmul)(b,lb,a,l,d); else xn(fftmul)(a,l,b,lb,d);
   xn(dec1)(d,la);
@@ -341,7 +377,7 @@ void xn(calc_moddivr)
 (chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiffre *d) {
   long l = la-lb;
 
-  b[lb-1] |= (BASE/2);
+  b[lb-1] |= (BASE_2);
   if (l > 2) xn(clear)(a,l/2);
   if (l < lb) xn(fftmul)(b,lb,a,l,d); else xn(fftmul)(a,l,b,lb,d);
   xn(dec1)(d,la);
@@ -355,7 +391,7 @@ void xn(calc_moddivr)
 void xn(calc_karpinv)
 (chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiffre *d) {
   
-  a[la-1] |= (BASE/2);
+  a[la-1] |= (BASE_2);
   xn(karpinv)(a,la,b);
 }
 
@@ -364,8 +400,8 @@ void xn(calc_karpinv)
 void xn(calc_karpdivnr)
 (chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiffre *d) {
   
-  b[lb-1] |= (BASE/2);
-  if ((ndouble)a[la-1] >= (ndouble)b[lb-1]) a[la-1] = b[lb-1] - 1;
+  b[lb-1] |= (BASE_2);
+  if (a[la-1] >= b[lb-1]) a[la-1] = b[lb-1] - 1;
   xn(move)(a,la,d);
   xn(karpdiv)(d,la-lb,b,lb,c,0);
 }
@@ -373,8 +409,8 @@ void xn(calc_karpdivnr)
 void xn(calc_karpdivr)
 (chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiffre *d) {
   
-  b[lb-1] |= (BASE/2);
-  if ((ndouble)a[la-1] >= (ndouble)b[lb-1]) a[la-1] = b[lb-1] - 1;
+  b[lb-1] |= (BASE_2);
+  if (a[la-1] >= b[lb-1]) a[la-1] = b[lb-1] - 1;
   xn(move)(a,la,d);
   xn(karpdiv)(d,la-lb,b,lb,c,1);
 }
@@ -385,8 +421,8 @@ void xn(calc_karpdivnr)
 (chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiffre *d) {
   long l=la-lb;
   
-  b[lb-1] |= (BASE/2);
-  if ((ndouble)a[la-1] >= (ndouble)b[lb-1]) a[la-1] = b[lb-1] - 1;
+  b[lb-1] |= (BASE_2);
+  if (a[la-1] >= b[lb-1]) a[la-1] = b[lb-1] - 1;
   if (l > 2)  xn(clear)(a,l/2);
   if (l < lb) xn(fftmul)(b,lb,a,l,d); else xn(fftmul)(a,l,b,lb,d);
   xn(dec1)(d,la);
@@ -397,8 +433,8 @@ void xn(calc_karpdivr)
 (chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiffre *d) {
   long l=la-lb;
   
-  b[lb-1] |= (BASE/2);
-  if ((ndouble)a[la-1] >= (ndouble)b[lb-1]) a[la-1] = b[lb-1] - 1;
+  b[lb-1] |= (BASE_2);
+  if (a[la-1] >= b[lb-1]) a[la-1] = b[lb-1] - 1;
   if (l > 2)  xn(clear)(a,l/2);
   if (l < lb) xn(fftmul)(b,lb,a,l,d); else xn(fftmul)(a,l,b,lb,d);
   xn(dec1)(d,la);
@@ -427,8 +463,8 @@ void xn(size_sqrt)(long *la, long *lb, long *lc, long *ld) {
 void xn(calc_sqrt_n2)
 (chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiffre *d) {
   chiffre x;
-  x = a[la-1] & (BASE/4 - 1);
-  if (x < BASE/16) x += BASE/16;
+  x = a[la-1] & (BASE_2/2 - 1);
+  if (x < BASE_2/8) x += BASE_2/8;
   a[la-1] = x;
   xn(move)(a,la,c);
   xn(sqrt_n2)(c,la,b);
@@ -440,8 +476,8 @@ void xn(calc_sqrt_n2)
 (chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiffre *d) {
   chiffre x;
 
-  x = a[la/2-1] & (BASE/2 - 1);
-  if (x < BASE/4) x += BASE/4;
+  x = a[la/2-1] & (BASE_2 - 1);
+  if (x < BASE_2/2) x += BASE_2/2;
   a[la/2-1] = x;
   xn(toomsqr)(a,la/2,c);
   xn(dec1)(c,la);
@@ -457,8 +493,8 @@ void xn(calc_sqrt_n2)
 void xn(calc_zimsqrt)
 (chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiffre *d) {
   chiffre x;
-  x = a[la-1] & (BASE/4 - 1);
-  if (x < BASE/16) x += BASE/16;
+  x = a[la-1] & (BASE_2/2 - 1);
+  if (x < BASE_2/8) x += BASE_2/8;
   a[la-1] = x;
   xn(move)(a,la,c);
   xn(zimsqrt)(c,la,b);
@@ -471,8 +507,8 @@ void xn(calc_zimsqrt)
 (chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiffre *d) {
   chiffre x;
 
-  x = a[la/2-1] & (BASE/2 - 1);
-  if (x < BASE/4) x += BASE/4;
+  x = a[la/2-1] & (BASE_2 - 1);
+  if (x < BASE_2/2) x += BASE_2/2;
   a[la/2-1] = x;
   xn(toomsqr)(a,la/2,c);
   xn(dec1)(c,la);
@@ -488,8 +524,8 @@ void xn(calc_zimsqrt)
 void xn(calc_modsqrt)
 (chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiffre *d) {
   chiffre x;
-  x = a[la-1] & (BASE/4 - 1);
-  if (x < BASE/16) x += BASE/16;
+  x = a[la-1] & (BASE_2/2 - 1);
+  if (x < BASE_2/8) x += BASE_2/8;
   a[la-1] = x;
   xn(move)(a,la,c);
   xn(modsqrt)(c,la,b);
@@ -502,8 +538,8 @@ void xn(calc_modsqrt)
 (chiffre *a, long la, chiffre *b, long lb, chiffre *c, chiffre *d) {
   chiffre x;
 
-  x = a[la/2-1] & (BASE/2 - 1);
-  if (x < BASE/4) x += BASE/4;
+  x = a[la/2-1] & (BASE_2 - 1);
+  if (x < BASE_2/2) x += BASE_2/2;
   a[la/2-1] = x;
   if (la > 8) {xn(clear)(a,la/4);xn(dec1)(a,la/2);}
   xn(fftsqr)(a,la/2,c);
@@ -690,6 +726,10 @@ test_t test[] = {
 {0, "but2",        "butterfly, q=2*HW+1",          xn(size_but),   xn(calc_but2)     },
 {0, "but3",        "butterfly, q=2*HW+2",          xn(size_but),   xn(calc_but3)     },
 
+/* sjoin3 */
+{0, NULL,          "sjoin3",                       NULL,           NULL              },
+{0, "join",        "sjoin3",                       xn(size_join),   xn(calc_join)     },
+ 
 /* multiplication */
 {0, NULL,          "Multiplication n by n",        NULL,           NULL              },
 {0, "mul_n2",      "schoolbook multiplication",    xn(size_mul),   xn(calc_mul_n2)   },
@@ -850,10 +890,10 @@ void chronometre(int t, long *m, long *n, double temps, long bits, double unit)
     if (t1 < 0) {
       /* dbordement de l'horloge. Avec un peu de chance a n'a dbord
          qu'une fois, */
-      t1 += 1 << (8*sizeof(clock_t)-2);
-      t1 += 1 << (8*sizeof(clock_t)-2);
-      t1 += 1 << (8*sizeof(clock_t)-2);
-      t1 += 1 << (8*sizeof(clock_t)-2);
+      t1 += 1L << (8*sizeof(clock_t)-2);
+      t1 += 1L << (8*sizeof(clock_t)-2);
+      t1 += 1L << (8*sizeof(clock_t)-2);
+      t1 += 1L << (8*sizeof(clock_t)-2);
     }
 
     /* libre la mmoire */
@@ -891,9 +931,9 @@ void read_hexstring(chiffre *a, long la, char *s) {
     if (s) {
         xn(clear)(a,la);
         for (i=0, l=strlen(s), s+=l-1; (l) && (la); i+=4, s--, l--) {
-            if      ((s[0] >= '0') && (s[0] <= '9')) a[0] += (s[0] - '0'     ) << i;
-            else if ((s[0] >= 'A') && (s[0] <= 'F')) a[0] += (s[0] - 'A' + 10) << i;
-            else if ((s[0] >= 'a') && (s[0] <= 'f')) a[0] += (s[0] - 'a' + 10) << i;
+            if      ((s[0] >= '0') && (s[0] <= '9')) a[0] += (chiffre)(s[0] - '0'     ) << i;
+            else if ((s[0] >= 'A') && (s[0] <= 'F')) a[0] += (chiffre)(s[0] - 'A' + 10) << i;
+            else if ((s[0] >= 'a') && (s[0] <= 'f')) a[0] += (chiffre)(s[0] - 'a' + 10) << i;
             else {
                 printf("invalid hexadecimal string\n");
                 fflush(stdout);
@@ -1073,7 +1113,7 @@ int main(int argc, char **argv) {
   }
 
   /* initialisation des gnrateurs alatoires */
-  srand(seed);
+  srandom(seed);
 
   /* ouvre le fichier de sortie en mode append */
   if (res) {
@@ -1085,23 +1125,25 @@ int main(int argc, char **argv) {
       }
   }
 
-#define PRINTF(...) do {printf(__VA_ARGS__); if (res) fprintf(fres,__VA_ARGS__);} while (0)
-
   /* tests pour a et/ou b impos */
   if ((sa) || (sb)) {
     for (i=0; i < nbtests; i++) if (test[i].sel) {
-        PRINTF("%-16s", test[i].name);
+        printf("%-16s", test[i].name);
+        if (res) fprintf(fres,"%-16s", test[i].name);
         single_run(i,sa,sb);
-        PRINTF("ok\n");
+        printf("ok\n");
+        if (res) fprintf(fres,"ok\n");
     }
   }
 
   /* tests pour une seule longueur */
   else if (!range) {
     for (i=0; i < nbtests; i++) if (test[i].sel) {
-      PRINTF("%-16s", test[i].name);
+      printf("%-16s", test[i].name);
+      if (res) fprintf(fres,"%-16s", test[i].name);
       chronometre(i,&m,&n,temps,bits,unit);
-      PRINTF("%10ld (%ld)\n", m,n);
+      printf("%10ld (%ld)\n", m,n);
+      if (res) fprintf(fres,"%10ld (%ld)\n", m,n);
     }
   }
 
@@ -1109,26 +1151,41 @@ int main(int argc, char **argv) {
   else {
 
     /* recopie la ligne de commande */
-    PRINTF("#");
-    for (j=argc, s=argv; j; j--, s++) PRINTF(" %s",*s);
-    PRINTF("\n"); fflush(stdout);
+    printf("#");
+    for (j=argc, s=argv; j; j--, s++) printf(" %s",*s);
+    printf("\n"); fflush(stdout);
+    if (res) {
+      fprintf(fres,"#");
+      for (j=argc, s=argv; j; j--, s++) fprintf(fres," %s",*s);
+      fprintf(fres,"\n");
+    }
 
     /* noms des algorithmes */
-    PRINTF("#           bits");
-    for (i=0; i < nbtests; i++) if (test[i].sel) PRINTF("%16s", test[i].name);
-    PRINTF("\n");
+    printf("#           bits");
+    for (i=0; i < nbtests; i++) if (test[i].sel) printf("%16s", test[i].name);
+    printf("\n");
+    if (res) {
+      fprintf(fres,"#           bits");
+      for (i=0; i < nbtests; i++) if (test[i].sel) fprintf(fres,"%16s", test[i].name);
+      fprintf(fres,"\n");
+    }
 
     for (bits = start; bits <= stop; bits += step) {
-      PRINTF("%16ld", bits);
+      printf("%16ld", bits);
+      if (res) fprintf(fres,"%16ld", bits);
       for (i=0; i < nbtests; i++) if (test[i].sel) {
         chronometre(i,&m,&n,temps,bits,unit);
-        PRINTF("%16ld",m);
+        printf("%16ld",m);
+        if (res) fprintf(fres,"%16ld",m);
       }
-      PRINTF("\n");
+      printf("\n");
+      if (res) fprintf(fres,"\n");
     }
   }
 
   /* termin */
+  fflush(stdout);
+  if (res) {fflush(fres); fclose(fres);}
   return(0);
 }
 
