@@ -1,6 +1,6 @@
 // file kernel/ocaml/c/gmp.c: Interface with Gmp
 /*-----------------------------------------------------------------------+
- |  Copyright 2005, Michel Quercia (michel.quercia@prepas.org)           |
+ |  Copyright 2005-2006, Michel Quercia (michel.quercia@prepas.org)      |
  |                                                                       |
  |  This file is part of Numerix. Numerix is free software; you can      |
  |  redistribute it and/or modify it under the terms of the GNU Lesser   |
@@ -54,6 +54,9 @@ extern value caml_check_urgent_gc (value);
 /* mode d'arrondi */
 #define Round_val Long_val
 
+/* valeur 3 tats */
+#define Val_tri Val_long
+
 /* Un entier est vu depuis Ocaml comme une donne abstraite
    avec customisation */
 typedef struct {
