@@ -1,6 +1,6 @@
 (* file kernel/ocaml/ml/big.ml: Objective-Caml bignum interface
  +-----------------------------------------------------------------------+
- |  Copyright 2005, Michel Quercia (michel.quercia@prepas.org)           |
+ |  Copyright 2005-2006, Michel Quercia (michel.quercia@prepas.org)      |
  |                                                                       |
  |  This file is part of Numerix. Numerix is free software; you can      |
  |  redistribute it and/or modify it under the terms of the GNU Lesser   |
@@ -492,4 +492,148 @@ module Big = struct
     toplevel_print(look a);
     Format.print_char ')'
 
+                         (* +---------------------+
+                            |  Test de primalit  |
+                            +---------------------+ *)
+
+  (* n >= 0, cherche s'il a un diviseur non trivial <= 2^10 *)
+  let trivial_prime n =
+
+    let rec loop d =
+      if inf_1 n (d*d)      then True
+      else if d > 1024      then Unknown
+      else if mod_1 n d = 0 then False
+      else loop(d+2)
+    in
+
+    match cmp_1 n 2 with
+    |  1 when nth_bit n 0 -> loop 3
+    |  0 -> True
+    |  _ -> False
+
+  (* test de Rabin-Miller pour les bases 2,3,5,7,11,13,17.
+     on suppose n >= 0 et trivial_prime(n) = Unknown
+     (ie. n >= 2^20 et n n'a pas de facteur premier <= 2^10)
+
+     Ribenboim, The Little Book of Bigger Primes, ch. VII p. 98 :
+
+     Given k >= 1, denote by tk the smallest integer which is a strong pseudoprime
+     for the bases p1 = 2, p2 = 3, . . . , pk, simultaneously. Then the
+     calculations of Pomerance, Selfridge & Wagstaff (1980), extended by
+     Jaeschke (1993), provide the following values:
+
+     t2 =                1 373 653 = 829  1657,
+     t3 =               25 326 001 = 2251  11251,
+     t4 =            3 215 031 751 = 151  751  28351,
+     t5 =        2 152 302 898 747 = 6763  10627  29947,
+     t6 =        3 474 749 660 383 = 1303  16927  157543,
+     t7 = t8 = 341 550 071 728 321 = 10670053  32010157.
+
+     Complment (http://primes.utm.edu/prove/prove2_3.html) : le premier
+     contre-exemple suivant t4 est
+
+     t4' = 118 670 087 467 = 172243  688969.
+
+     On utilisera cette borne car t4 est reconnu non premier par trivial_prime.
+     Le mme argument s'applique  t2, avec
+
+     t2' = 2 284 453 = 1069  2137.
+
+     Le premier nombre sans facteur trivial spp(2) est
+
+     t1' = 1 194 649 = 1093^2.
+
+     Quelques contre-exemples (thse de Franois Arnault,
+     http://www.unilim.fr/pages_perso/francois.arnault/documents/these.ps):
+
+     1195068768795265792518361315725116351898245581
+     = 24444516448431392447461  48889032896862784894921
+     est spp(2,..,31)
+
+     80383745745363949125707961434194210813883768828755\
+     81458374889175222974273765333652186502336163960045\
+     45791504202360320876656996676098728404396540823292\
+     87387918508691668573282677617710293896977394701670\
+     82304286871099974399765441448453411558724506334092\
+     79022275296229414984230688168540432645753401832978\
+     6111298960644845216191652872597534901
+
+     = 20047910831974980270915322604227342650259408302\
+     05662543872531023690016085350598121358111595798609\
+     86679108158254267908348457261690695858464376399022\
+     2898400226296015918301
+
+      400958216639499605418306452084546853005188166041\
+     13250877450620473800321707011962427162231915972197\
+     33582163165085358166969145233813917169287527980445\
+     796800452592031836601
+
+     est spp(2,..,199)
+
+     Ces deux nombres sont reconnus non premiers par small_prime en comparant les
+     racines carres de -1 obtenues aux cours des tests. De mme, tous les spp
+     donns dans la thse de Franois Arnault page 29 sont reconnus non premiers
+     par le mme critre  l'exception de
+
+     n = 340770619658554299579514119954792724101734599955221
+     = 13053172404794060245721341  26106344809588120491442681
+
+     qui est spp(2,..,31). On aurait trouv une deuxime racine carre
+     de -1 si on avait pouss le test jusqu' la base 23.
+  *)
+
+  let miller_rabin =
+
+    let tests = [of_int  2, of_string         "1194649";
+                 of_int  3, of_string         "2284453";
+		 of_int  5, of_string        "25326001";
+		 of_int  7, of_string    "118670087467";
+		 of_int 11, of_string   "2152302898747";
+		 of_int 13, of_string   "3474749660383";
+		 of_int 17, of_string "341550071728321"] in
+
+    fun n ->
+
+      (* dcompose n-1 = q*2^k, q impair *)
+      let q,k =
+	let rec loop k = if nth_bit n k then (shr n k),k else loop (k+1)
+	in loop 1
+      in
+
+      (* racine de -1 potentielle *)
+      let i = make_ref(zero) in
+
+      (* lve a au carr au plus k-2 fois jusqu' trouver une racine carre de -1 *)
+      let rec sqr_loop a k =
+	if k <= 1 then None
+	else let b = gmod Nearest_up (sqr a) n in
+	     if eq_1 b (-1) then Some(if sgn a > 0 then a else neg a)
+	     else sqr_loop b (k-1)
+      in
+
+      (* test de Rabin-Miller pour la base a *)
+      let mr_test a =
+	let b = gpowmod Nearest_up a q n in
+	(eq_1 b 1) or (eq_1 b (-1)) or match sqr_loop b k with
+	| Some(x) when eq_1 (look i) 0 -> copy_in i x; true
+	| Some(x) -> eq (look i) x
+	| None    -> false
+      in
+
+      (* passe en revue toutes les bases jusqu' obtenir une certitude *)
+      let rec loop l = match l with
+      | [] -> Unknown
+      | (a,t)::suite -> match cmp t n with
+	  | 0 -> False
+	  | 1 -> if mr_test a then True else False
+	  | _ -> if mr_test a then loop suite else False
+      in loop tests
+
+  (* test de primalit *)
+  let isprime n =
+    let n = if inf_1 n 0 then neg n else n in
+    match trivial_prime n with Unknown -> miller_rabin n | r -> r
+
+  let isprime_1 n = isprime(of_int n)
+
 end
