@@ -1,6 +1,6 @@
 (* file kernel/ocaml/ml/cmp.ml: Comparison between two modules
  +-----------------------------------------------------------------------+
- |  Copyright 2005, Michel Quercia (michel.quercia@prepas.org)           |
+ |  Copyright 2005-2006, Michel Quercia (michel.quercia@prepas.org)      |
  |                                                                       |
  |  This file is part of Numerix. Numerix is free software; you can      |
  |  redistribute it and/or modify it under the terms of the GNU Lesser   |
@@ -58,6 +58,12 @@ module Cmp(A:Int_type)(B:Int_type) = struct
     | Nearest_up   -> "Nearest_up"
     | Nearest_down -> "Nearest_down"
 
+  (* noms des rsultats tristate *)
+  let cstring_of = function
+    | False   -> "False"
+    | Unknown -> "Unknown"
+    | True    -> "True"
+
   (*
     comparaison et agrgation de rsultats calculs dans A et B :
     on compare smantiquement les rsultats et en cas d''accord on
@@ -73,6 +79,7 @@ module Cmp(A:Int_type)(B:Int_type) = struct
     round_mode = m
     string     = s
     bool       = b
+    tristate   = c
   *)
 
   exception Error of string
@@ -108,6 +115,11 @@ module Cmp(A:Int_type)(B:Int_type) = struct
       aname fn (mstring_of m) (A.string_of a1) (A.string_of a2) (A.string_of a3) (A.string_of a4)
       bname fn (mstring_of m) (B.string_of b1) (B.string_of b2) (B.string_of b3) (B.string_of b4)
 
+  let dif_i_c fn ia1 ca2 ib1 cb2 =
+    sprintf "%s.%s = (%d,%s) %s.%s = (%d,%s)"
+      aname fn ia1 (cstring_of ca2)
+      bname fn ib1 (cstring_of cb2)
+
   let dif_i_i_t fn ia1 ia2 a3 ib1 ib2 b3 =
     sprintf "%s.%s = (%d,%d,%s) %s.%s = (%d,%d,%s)"
       aname fn ia1 ia2 (A.string_of a3)
@@ -128,6 +140,11 @@ module Cmp(A:Int_type)(B:Int_type) = struct
       aname fn sa1 (A.string_of a2)
       bname fn sb1 (B.string_of b2)
 
+  let dif_t_c fn a1 ca2 b1 cb2 =
+    sprintf "%s.%s = (%s,%s) %s.%s = (%s,%s)"
+      aname fn (A.string_of a1) (cstring_of ca2)
+      bname fn (B.string_of b1) (cstring_of cb2)
+
   let dif_t_i fn a1 ia2 b1 ib2 =
     sprintf "%s.%s = (%s,%d) %s.%s = (%s,%d)"
       aname fn (A.string_of a1) ia2
@@ -721,6 +738,17 @@ module Cmp(A:Int_type)(B:Int_type) = struct
     end
     else raise (Error (dif_t_t_t_t_t_t_t "cfrac_in" aa6 aa7 a1 a2 a3 a4 a5 bb6 bb7 b1 b2 b3 b4 b5))
 
+  (* test de primalit : on prend le rsultat le plus prcis *)
+  let isprime (a,b) = match (A.isprime a),(B.isprime b) with
+  | Unknown,r -> r
+  | r,Unknown -> r
+  | r,s       -> if r = s then r else raise (Error (dif_t_c "isprime" a r b s))
+
+  let isprime_1 i = match (A.isprime_1 i),(B.isprime_1 i) with
+  | Unknown,r -> r
+  | r,Unknown -> r
+  | r,s       -> if r = s then r else raise (Error (dif_i_c "isprime_1" i r i s))
+
   (* nombres alatoires : on prend le gnrateur de A *)
   let nrandom  n = let a = A.nrandom  n in a,(b_of_a a)
   let zrandom  n = let a = A.zrandom  n in a,(b_of_a a)
