@@ -1,27 +1,27 @@
 (* file kernel/ocaml/ml/rfuns.ml: Approximation of usual real-valued functions
-   +-----------------------------------------------------------------------+
-   |  Copyright 2005, Michel Quercia (michel.quercia@prepas.org)           |
-   |                                                                       |
-   |  This file is part of Numerix. Numerix is free software; you can      |
-   |  redistribute it and/or modify it under the terms of the GNU Lesser   |
-   |  General Public License as published by the Free Software Foundation; |
-   |  either version 2.1 of the License, or (at your option) any later     |
-   |  version.                                                             |
-   |                                                                       |
-   |  The Numerix Library is distributed in the hope that it will be       |
-   |  useful, but WITHOUT ANY WARRANTY; without even the implied warranty  |
-   |  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  |
-   |  Lesser General Public License for more details.                      |
-   |                                                                       |
-   |  You should have received a copy of the GNU Lesser General Public     |
-   |  License along with the GNU MP Library; see the file COPYING. If not, |
-   |  write to the Free Software Foundation, Inc., 59 Temple Place -       |
-   |  Suite 330, Boston, MA 02111-1307, USA.                               |
-   +-----------------------------------------------------------------------+
-   |                                                                       |
-   |         Approximation des fonctions usuelles  valeurs relles        |
-   |                                                                       |
-   +-----------------------------------------------------------------------*)
+ +-----------------------------------------------------------------------+
+ |  Copyright 2005-2006, Michel Quercia (michel.quercia@prepas.org)      |
+ |                                                                       |
+ |  This file is part of Numerix. Numerix is free software; you can      |
+ |  redistribute it and/or modify it under the terms of the GNU Lesser   |
+ |  General Public License as published by the Free Software Foundation; |
+ |  either version 2.1 of the License, or (at your option) any later     |
+ |  version.                                                             |
+ |                                                                       |
+ |  The Numerix Library is distributed in the hope that it will be       |
+ |  useful, but WITHOUT ANY WARRANTY; without even the implied warranty  |
+ |  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  |
+ |  Lesser General Public License for more details.                      |
+ |                                                                       |
+ |  You should have received a copy of the GNU Lesser General Public     |
+ |  License along with the GNU MP Library; see the file COPYING. If not, |
+ |  write to the Free Software Foundation, Inc., 59 Temple Place -       |
+ |  Suite 330, Boston, MA 02111-1307, USA.                               |
+ +-----------------------------------------------------------------------+
+ |                                                                       |
+ |         Approximation des fonctions usuelles  valeurs relles        |
+ |                                                                       |
+ +-----------------------------------------------------------------------*)
 
       (* +----------------------------------------------------------+
          |  Approximation des fonctions usuelles  valeurs relles  |
@@ -42,6 +42,10 @@ module Rfuns(E:Int_type) = struct
                                |  Utilitaires  |
                                +---------------+ *)
 
+  (* cration d'une fonction e_f *)
+  let emake f =
+    fun a p n -> if p >= 0 then f a (shl one p) n else f (shr a p) one n
+
   (* partie entire suprieure de i/j, i,j = entiers courts avec j > 0 *)
   let ceil i j = let x = i/j in if i mod j > 0 then x+1 else x
 
@@ -55,6 +59,9 @@ module Rfuns(E:Int_type) = struct
   let ( >> ) a n =
     let x = shr a n in if nth_bit a (n-1) then x +. sgn(a) else x
 
+  (* valeur absolue sans recopie inutile *)
+  let abs a = if sgn(a) < 0 then neg a else a
+
                     (* +------------------------------+
                        |  Mmorisation de constantes  |
                        +------------------------------+ *)
@@ -73,7 +80,7 @@ module Rfuns(E:Int_type) = struct
   (* Pour obtenir une approximation de x  p bits :
 
      1. Si p > x.n, recalculer une valeur plus prcise par x.calc(q) avec q >= p.
-     Cette instruction modifie les champs x.c,x.n,x.d1 et x.d2 de sorte qu'on
+     Cette instruction modifie les champs x.c,x.n et x.d de sorte qu'on
      dispose  prsent de q bits. En particulier x.n = q.
 
      2. Retourner (x.c + (x.d mod 2^(k+1))) / 2^k avec k = x.n - p.
@@ -113,7 +120,7 @@ module Rfuns(E:Int_type) = struct
       in
         x.c <- c;
         x.n <- n;
-        x.d <- (x.d  ++ d) << p
+        x.d <- (x.d ++ d) << p
     in
 
     (* recalcul avec q bits *)
@@ -194,28 +201,28 @@ module Rfuns(E:Int_type) = struct
 
     if n < 0 then raise bit_overflow;
 
-      (* calcule c*f(n) puis ajoute 1/2 (Nearest_xxx) ou 1 (Ceil)
-         l'arrondi demand est la partie entire du rsultat, si elle peut
-         tre dtermine.
-      *)
-      let x = c**f(n) in
-      let x = match r with
-        | Floor -> x
-        | Ceil  -> x ++ (one << n)
-        | _     -> x ++ (one << (n-1)) in
+    (* calcule c*f(n) puis ajoute 1/2 (Nearest_xxx) ou 1 (Ceil)
+       l'arrondi demand est la partie entire du rsultat, si elle peut
+       tre dtermine.
+    *)
+    let x = c**f(n) in
+    let x = match r with
+      | Floor -> x
+      | Ceil  -> x ++ (one << n)
+      | _     -> x ++ (one << (n-1)) in
 
-      let x,y = split x n in
-      let y,z = match sgn(c) with 1 -> y--c, y++c | _ -> y++c, y--c in
+    let x,y = split x n in
+    let y,z = match sgn(c) with 1 -> y--c, y++c | _ -> y++c, y--c in
 
-        (* on a  x + y/2^n < res < x + z/2^n, y/2^n < 1 et z/2^n > -1
+    (* on a  x + y/2^n < res < x + z/2^n, y/2^n < 1 et z/2^n > -1
 
-           donc si    0 <= y/2^n  et  z/2^n <= 1  alors floor = x
-                si   -1 <= y/2^n  et  z/2^n <= 0  alors floor = x-1
-                sinon, il faut recommencer avec plus de bits
-        *)
-             if (y >=. 0) & not (nth_bit (z-.1) n) then x
-        else if (z <=. 0) & not (nth_bit (y+.1) n) then x-.1
-        else round_loop f r c (n+p) (2*p)
+       donc si    0 <= y/2^n  et  z/2^n <= 1  alors floor = x
+            si   -1 <= y/2^n  et  z/2^n <= 0  alors floor = x-1
+            sinon, il faut recommencer avec plus de bits
+    *)
+         if (y >=. 0) & not (nth_bit (z-.1) n) then x
+    else if (z <=. 0) & not (nth_bit (y+.1) n) then x-.1
+    else round_loop f r c (n+p) (2*p)
 
 
   (* f = a b n -> approx(2^n*f(a/b))
@@ -302,7 +309,7 @@ module Rfuns(E:Int_type) = struct
      n = nombre de bits dsirs
 
      Sortie :
-     retourne approx(2^n*argth(a/b) (s=false) ou approx(arctan(a/b) (s=true)
+     retourne approx(2^n*argth(a/b)) (s=false) ou approx(arctan(a/b)) (s=true)
 
      Algorithme de calcul :
      La somme de la srie entire est calcule terme  terme avec
@@ -408,13 +415,13 @@ module Rfuns(E:Int_type) = struct
       else if n >= max_int - 2 then raise bit_overflow
       else begin
         let na = nbits(a) and nb = nbits(b) in
-          if 7*na <= 4*nb then compute a b na nb n
-          else begin
-            let q,r = gquomod Nearest_up b a           in
-            let x = compute one q 1 (nbits q) (n+2)    in
-            let c = if s then q**b ++ a else q**b -- a in
-            let y = accel r c (n+2)                    in (x -- y) >> 2
-          end
+        if 7*na <= 4*nb then compute a b na nb n
+        else begin
+          let q,r = gquomod Nearest_up b a           in
+          let x = compute one q 1 (nbits q) (n+2)    in
+          let c = if s then q**b ++ a else q**b -- a in
+          let y = accel r c (n+2)                    in (x -- y) >> 2
+        end
       end
 
     in accel
@@ -612,6 +619,7 @@ module Rfuns(E:Int_type) = struct
     else                          Irrationnal
 
   let r_ln = round ln exact_ln
+  let e_ln = emake ln
 
                 (* +--------------------------------------+
                    |  Calcul de argth(a/b), arccoth(a/b)  |
@@ -642,7 +650,9 @@ module Rfuns(E:Int_type) = struct
   let exact_arccoth a b = exact_arctanh b a
 
   let r_arctanh = round arctanh exact_arctanh
+  let e_arctanh = emake arctanh
   let r_arccoth = round arccoth exact_arccoth
+  let e_arccoth = emake arccoth
 
 
                        (* +------------------------+
@@ -683,6 +693,7 @@ module Rfuns(E:Int_type) = struct
     else                Irrationnal
 
   let r_arcsinh = round arcsinh exact_arcsinh
+  let e_arcsinh = emake arcsinh
 
                        (* +------------------------+
                           |  Calcul de argch(a/b)  |
@@ -720,6 +731,7 @@ module Rfuns(E:Int_type) = struct
       else                     Irrationnal
 
   let r_arccosh = round arccosh exact_arccosh
+  let e_arccosh = emake arccosh
 
 
                         (* +-----------------------+
@@ -931,6 +943,7 @@ module Rfuns(E:Int_type) = struct
     if (a >. 0) & (b =. 0) then Exact(0,1) else Irrationnal
 
   let r_arg = round arg exact_arg
+  let e_arg = emake arg
 
 
                 (* +--------------------------------------+
@@ -961,7 +974,9 @@ module Rfuns(E:Int_type) = struct
     if (a >. 0) & (b =. 0) then Exact(0,1) else Irrationnal
 
   let r_arctan = round arctan exact_arctan
+  let e_arctan = emake arctan
   let r_arccot = round arccot exact_arccot
+  let e_arccot = emake arccot
 
 
                 (* +--------------------------------------+
@@ -1001,6 +1016,7 @@ module Rfuns(E:Int_type) = struct
     else                             Irrationnal
 
   let r_arcsin = round arcsin exact_arcsin
+  let e_arcsin = emake arcsin
 
   let arccos a b n =
     let a,b = match sgn(b) with
@@ -1008,16 +1024,13 @@ module Rfuns(E:Int_type) = struct
       | -1 -> (neg a),(neg b)
       |  _ -> raise (if a =. 0 then zero_zero else Error "arccos")
     in
-
-    if n <= -2 then zero
+    let b2 = sqr(b)       in
+    let c2 = b2 -- sqr(a) in
+    if c2 <. 0 then raise (Error "arccos")
+    else if n <= -2 then zero
     else if n >= max_int/2 -1 then raise bit_overflow
-    else begin
-      let b2 = sqr(b)       in
-      let c2 = b2 -- sqr(a) in
-      if c2 <. 0 then raise (Error "arccos");
-      let k = max 0 (n+3 + nbits(a) - nbits(b2)) in
-      (arg (a<<k) (gsqrt Ceil(c2 << (2*k))) (n+2)) >> 2
-    end
+    else let k = max 0 (n+3 + nbits(a) - nbits(b2))
+         in (arg (a<<k) (gsqrt Ceil(c2 << (2*k))) (n+2)) >> 2
 
   let exact_arccos a b =
          if eq a b              then Exact(0,1)
@@ -1025,6 +1038,8 @@ module Rfuns(E:Int_type) = struct
     else                             Irrationnal
 
   let r_arccos = round arccos exact_arccos
+  let e_arccos = emake arccos
+
 
                  (* +-------------------------------------+
                     |  Sommation de la srie entire exp  |
@@ -1351,6 +1366,7 @@ module Rfuns(E:Int_type) = struct
     else                             Irrationnal
 
   let r_exp = round exp exact_exp
+  let e_exp = emake exp
 
                     (* +------------------------------+
                        |  Calcul de ch(a/b), sh(a/b)  |
@@ -1379,8 +1395,11 @@ module Rfuns(E:Int_type) = struct
     if a =. 0 then Exact(0,1) else Irrationnal
 
   let r_cosinh = round2 cosinh exact_cosinh
+  let e_cosinh = emake  cosinh
   let r_cosh   = round  cosh   exact_cosh
+  let e_cosh   = emake  cosh
   let r_sinh   = round  sinh   exact_sinh
+  let e_sinh   = emake  sinh
 
 
                          (* +---------------------+
@@ -1428,6 +1447,7 @@ module Rfuns(E:Int_type) = struct
     else                Irrationnal
 
   let r_tanh = round tanh exact_tanh
+  let e_tanh = emake tanh
 
 
                         (* +-----------------------+
@@ -1477,6 +1497,7 @@ module Rfuns(E:Int_type) = struct
     else                Irrationnal
 
   let r_coth = round coth exact_coth
+  let e_coth = emake coth
 
 
               (* +------------------------------------------+
@@ -1683,8 +1704,11 @@ module Rfuns(E:Int_type) = struct
     if a =. 0 then Exact(0,1) else Irrationnal
 
   let r_cosin = round2 cosin exact_cosin
+  let e_cosin = emake  cosin
   let r_cos   = round  cos   exact_cos
+  let e_cos   = emake  cos
   let r_sin   = round  sin   exact_sin
+  let e_sin   = emake  sin
 
                   (* +----------------------------------+
                      |  Calcul de tan(a/b), cotan(a/b)  |
@@ -1745,7 +1769,9 @@ module Rfuns(E:Int_type) = struct
     if (a =. 0) or (b =. 0) then Invalid "cot" else Irrationnal
 
   let r_tan = round tan exact_tan
+  let e_tan = emake tan
   let r_cot = round cot exact_cot
+  let e_cot = emake cot
 
 
                     (* +------------------------------+
