@@ -1,6 +1,6 @@
 // file kernel/x/c/div.c: division of extensible integers
 /*-----------------------------------------------------------------------+
- |  Copyright 2005, Michel Quercia (michel.quercia@prepas.org)           |
+ |  Copyright 2005-2006, Michel Quercia (michel.quercia@prepas.org)      |
  |                                                                       |
  |  This file is part of Numerix. Numerix is free software; you can      |
  |  redistribute it and/or modify it under the terms of the GNU Lesser   |
@@ -236,7 +236,7 @@ xx(private_quomod)(xint *_c, xint *_d, xint a, xint b, long mode) {
         ld = lb;          if (want_d) d = xx(enlarge)(_d,ld);
 
         /* dcalage  appliquer pour avoir msb(b) = 1 */
-        for (r=b->val[lb-1], n=0; (r & (BASE/2)) == 0; r <<=1, n++);
+        for (r=b->val[lb-1], n=0; (r & (BASE_2)) == 0; r <<=1, n++);
 
         /* copie des oprandes :
            il faut copier b s'il va tre cras ou si n > 0
@@ -287,7 +287,7 @@ xx(private_quomod)(xint *_c, xint *_d, xint a, xint b, long mode) {
         }
 
         /* libre la mmoire temporaire */
-        xn(free)(x);
+        xn(free)(x); /* on peut avoir x = NULL ici */
     }
 
     /* ------------------------------ division sans reste  plusieurs chiffres */
@@ -301,7 +301,7 @@ xx(private_quomod)(xint *_c, xint *_d, xint a, xint b, long mode) {
         c = xx(enlarge)(_c,lc);
 
         /* dcalage  appliquer pour avoir msb(b) = 1 */
-        for (r=b->val[lb-1], n=0; (r & (BASE/2)) == 0; r <<=1, n++);
+        for (r=b->val[lb-1], n=0; (r & (BASE_2)) == 0; r <<=1, n++);
 
         /* copie des oprandes
            il faut copier b s'il va tre cras ou si n > 0
