@@ -1,6 +1,6 @@
 // file kernel/x/c/gmp.c: Caml/Ocaml interface with Gmp
 /*-----------------------------------------------------------------------+
- |  Copyright 2005, Michel Quercia (michel.quercia@prepas.org)           |
+ |  Copyright 2005-2006, Michel Quercia (michel.quercia@prepas.org)      |
  |                                                                       |
  |  This file is part of Numerix. Numerix is free software; you can      |
  |  redistribute it and/or modify it under the terms of the GNU Lesser   |
@@ -787,6 +787,25 @@ value gx_cfrac_bytecode(gint *v, int argn) {
     return(gx_cfrac(v[0],v[1],v[2],v[3],v[4],v[5],v[6]));
 }
 
+
+                              /* +-------------+
+                                 |  Primalit  |
+                                 +-------------+ */
+
+value gx_isprime(gint a) {
+  return(Val_tri(mpz_probab_prime_p(a->val,10)));
+}
+
+value gx_isprime_1(long a) {
+  mpz_t aa;
+  long r;
+  mpz_init_set_si(aa,Long_val(a));
+  r = Val_tri(mpz_probab_prime_p(aa,10));
+  mpz_clear(aa);
+  return(r);
+}
+
+
                            /* +----------------+
                               |  comparaisons  |
                               +----------------+ */
